from django.test import TestCase, RequestFactory, Client
import django
from django.core.urlresolvers import reverse
from django.db import models
from django.views.generic import ListView, CreateView , DetailView, UpdateView, DeleteView
from generic_scaffold import CrudManager, get_url_names
from generic_scaffold.templatetags.generic_scaffold_tags import set_urls_for_scaffold

class TestModel(models.Model):
    test = models.CharField(max_length=16)
    
class TestModel2(models.Model):
    test = models.CharField(max_length=16)
    
class TestEmptyModel(models.Model):
    test = models.CharField(max_length=16)

class TestModelImplicit(models.Model):
    test = models.CharField(max_length=16)
    
class TestModelExplicit(models.Model):
    test = models.CharField(max_length=16)

class TestCrudManager(CrudManager):
    model = TestModel
    prefix = 'test'
    
class TestEmptyPrefixCrudManager(CrudManager):
    model = TestEmptyModel

class TestImplicitCrudManager(CrudManager):
    model = TestModelImplicit
    prefix = 'test_implicit'

class TestExplicitCrudManager(CrudManager):
    model = TestModelExplicit
    prefix = 'test_explicit'
    list_template_name = 'generic_scaffold/list.html'
    form_template_name = 'generic_scaffold/form.html'
    detail_template_name = 'generic_scaffold/detail.html'
    delete_template_name = 'generic_scaffold/confirm_delete.html'


class TestOverrideViewsCrudManager(CrudManager):
    model = TestModel2
    prefix = 'test_override_views'
    
    list_view_class = type('OverridenListView', (ListView, ), {} )
    create_view_class = type('OverridenCreateView', (CreateView, ), {} )
    detail_view_class = type('OverridenDetailView', (DetailView, ), {} )
    update_view_class = type('OverridenUpdateView', (UpdateView, ), {} )
    delete_view_class = type('OverridenDeleteView', (DeleteView, ), {} )
    
    list_template_name = 'generic_scaffold/list.html'
    form_template_name = 'generic_scaffold/form.html'
    detail_template_name = 'generic_scaffold/detail.html'
    delete_template_name = 'generic_scaffold/confirm_delete.html'


test_crud = TestCrudManager()
urlpatterns = test_crud.get_url_patterns()

test_empty_prefix_crud = TestEmptyPrefixCrudManager()
urlpatterns += test_empty_prefix_crud.get_url_patterns()

test_implicit_crud = TestImplicitCrudManager()
urlpatterns += test_implicit_crud.get_url_patterns()

test_explicit_crud = TestExplicitCrudManager()
urlpatterns += test_explicit_crud.get_url_patterns()

test_override_crud = TestOverrideViewsCrudManager()
urlpatterns += test_override_crud.get_url_patterns()


class DuplicatesTest(TestCase):
    def test_duplicate_prefix(self):
        with self.assertRaises(django.core.exceptions.ImproperlyConfigured):
            klazz = type("Thrower", (CrudManager, ), {'prefix': 'test',} )
            
    def test_duplicate_model(self):
        with self.assertRaises(django.core.exceptions.ImproperlyConfigured):
            klazz = type("Thrower", (CrudManager, ), {
                'prefix': 'foo',
                'model': TestModel,
            } )
        

class EmptyPrefixTest(TestCase):
    def setUp(self):
        self.crud = test_empty_prefix_crud
        self.list_view = self.crud.get_list_class_view()
        self.create_view = self.crud.get_create_class_view()
        self.update_view = self.crud.get_update_class_view()
        self.delete_view = self.crud.get_delete_class_view()
        self.detail_view = self.crud.get_detail_class_view()

        TestEmptyModel.objects.create(test='test')

    def test_urls_have_correct_name(self):
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( getattr(self.crud, attr+'_url_name'), "generic_scaffold_testemptymodel_{0}".format(attr))

    def test_views_have_correct_parent_class(self):
        self.assertEquals(self.list_view.__bases__[-1].__name__, "ListView")
        self.assertEquals(self.create_view.__bases__[-1].__name__, "CreateView")
        self.assertEquals(self.update_view.__bases__[-1].__name__, "UpdateView")
        self.assertEquals(self.delete_view.__bases__[-1].__name__, "DeleteView")
        self.assertEquals(self.detail_view.__bases__[-1].__name__, "DetailView")

    def test_view_have_correct_model(self):
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( getattr(self, attr+'_view').model.__name__, "TestEmptyModel")

    def test_with_client(self):
        c = Client()

        list_resp = c.get( reverse(get_url_names(None)['list']))
        self.assertEquals(list_resp.status_code, 200)
        self.assertTrue(b'TestEmptyModel object' in list_resp.content)

        create_resp = c.get( reverse(get_url_names(None)['create']))
        self.assertEquals(create_resp.status_code, 200)
        self.assertTrue(b'id_test' in create_resp.content)

        update_resp = c.get( reverse(get_url_names(None)['update'], args=[1]))
        self.assertEquals(update_resp.status_code, 200)
        self.assertTrue(b'id_test' in update_resp.content)

        detail_resp = c.get( reverse(get_url_names(None)['detail'], args=[1]))
        self.assertEquals(detail_resp.status_code, 200)
        self.assertTrue(b'TestEmptyModel object' in detail_resp.content)

        delete_resp = c.get( reverse(get_url_names(None)['delete'], args=[1]))
        self.assertEquals(delete_resp.status_code, 200)
        self.assertTrue(b'TestEmptyModel object' in delete_resp.content)



class SimpleParameterTest(TestCase):
    def setUp(self):
        self.crud = test_crud
        self.list_view = self.crud.get_list_class_view()
        self.create_view = self.crud.get_create_class_view()
        self.update_view = self.crud.get_update_class_view()
        self.delete_view = self.crud.get_delete_class_view()
        self.detail_view = self.crud.get_detail_class_view()

        TestModel.objects.create(test='test')

    def test_urls_have_correct_name(self):
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( getattr(self.crud, attr+'_url_name'), "{0}_generic_scaffold_testmodel_{1}".format(TestCrudManager.prefix, attr))

    def test_views_have_correct_parent_class(self):
        self.assertEquals(self.list_view.__bases__[-1].__name__, "ListView")
        self.assertEquals(self.create_view.__bases__[-1].__name__, "CreateView")
        self.assertEquals(self.update_view.__bases__[-1].__name__, "UpdateView")
        self.assertEquals(self.delete_view.__bases__[-1].__name__, "DeleteView")
        self.assertEquals(self.detail_view.__bases__[-1].__name__, "DetailView")

    def test_view_have_correct_model(self):
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( getattr(self, attr+'_view').model.__name__, "TestModel")

    def test_with_client(self):
        c = Client()

        list_resp = c.get( reverse(get_url_names(prefix='test')['list']))
        self.assertEquals(list_resp.status_code, 200)
        self.assertTrue(b'TestModel object' in list_resp.content)

        create_resp = c.get( reverse(get_url_names(prefix='test')['create']))
        self.assertEquals(create_resp.status_code, 200)
        self.assertTrue(b'id_test' in create_resp.content)

        update_resp = c.get( reverse(get_url_names(prefix='test')['update'], args=[1]))
        self.assertEquals(update_resp.status_code, 200)
        self.assertTrue(b'id_test' in update_resp.content)

        detail_resp = c.get( reverse(get_url_names(prefix='test')['detail'], args=[1]))
        self.assertEquals(detail_resp.status_code, 200)
        self.assertTrue(b'TestModel object' in detail_resp.content)

        delete_resp = c.get( reverse(get_url_names(prefix='test')['delete'], args=[1]))
        self.assertEquals(delete_resp.status_code, 200)
        self.assertTrue(b'TestModel object' in delete_resp.content)


class TemplateOrderingTest(TestCase):
    def setUp(self):
        self.client = Client()
        TestModel.objects.create(test='test')
        TestModelImplicit.objects.create(test='test')
        TestModelExplicit.objects.create(test='test')

    def test_fallback_templates(self):
        list_resp = self.client.get( reverse(get_url_names(prefix='test')['list']))
        self.assertTemplateUsed(list_resp, 'generic_scaffold/list.html' )

        create_resp = self.client.get( reverse(get_url_names(prefix='test')['create']))
        self.assertTemplateUsed(create_resp, 'generic_scaffold/form.html' )

        update_resp = self.client.get( reverse(get_url_names(prefix='test')['update'], args=[1]))
        self.assertTemplateUsed(update_resp, 'generic_scaffold/form.html' )

        detail_resp = self.client.get( reverse(get_url_names(prefix='test')['detail'], args=[1]))
        self.assertTemplateUsed(detail_resp, 'generic_scaffold/detail.html' )

        delete_resp = self.client.get( reverse(get_url_names(prefix='test')['delete'], args=[1]))
        self.assertTemplateUsed(delete_resp, 'generic_scaffold/confirm_delete.html' )

    def test_implicit_templates(self):
        list_resp = self.client.get( reverse(get_url_names(prefix='test_implicit')['list']))
        self.assertTemplateUsed(list_resp, 'generic_scaffold/testmodelimplicit_list.html' )

        create_resp = self.client.get( reverse(get_url_names(prefix='test_implicit')['create']))
        self.assertTemplateUsed(create_resp, 'generic_scaffold/testmodelimplicit_form.html' )

        update_resp = self.client.get( reverse(get_url_names(prefix='test_implicit')['update'], args=[1]))
        self.assertTemplateUsed(update_resp, 'generic_scaffold/testmodelimplicit_form.html' )

        detail_resp = self.client.get( reverse(get_url_names(prefix='test_implicit')['detail'], args=[1]))
        self.assertTemplateUsed(detail_resp, 'generic_scaffold/testmodelimplicit_detail.html' )

        delete_resp = self.client.get( reverse(get_url_names(prefix='test_implicit')['delete'], args=[1]))
        self.assertTemplateUsed(delete_resp, 'generic_scaffold/testmodelimplicit_confirm_delete.html' )

    def test_explicit_templates(self):
        list_resp = self.client.get( reverse(get_url_names(prefix='test_explicit')['list']))
        self.assertTemplateUsed(list_resp, 'generic_scaffold/list.html' )

        create_resp = self.client.get( reverse(get_url_names(prefix='test_explicit')['create']))
        self.assertTemplateUsed(create_resp, 'generic_scaffold/form.html' )

        update_resp = self.client.get( reverse(get_url_names(prefix='test_explicit')['update'], args=[1]))
        self.assertTemplateUsed(update_resp, 'generic_scaffold/form.html' )

        detail_resp = self.client.get( reverse(get_url_names(prefix='test_explicit')['detail'], args=[1]))
        self.assertTemplateUsed(detail_resp, 'generic_scaffold/detail.html' )

        delete_resp = self.client.get( reverse(get_url_names(prefix='test_explicit')['delete'], args=[1]))
        self.assertTemplateUsed(delete_resp, 'generic_scaffold/confirm_delete.html' )


class TestUrlNames(TestCase):
    def setUp(self):
        pass
        
    def test_get_url_names_with_prefix(self):
        names = get_url_names(prefix='test')
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( names[attr], "{0}_generic_scaffold_testmodel_{1}".format(TestCrudManager.prefix, attr))
            
    def test_get_url_names_with_model(self):
        names = get_url_names(app='generic_scaffold', model='testmodel')
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( names[attr], "{0}_generic_scaffold_testmodel_{1}".format(TestCrudManager.prefix, attr))
            
class TestTempalteTags(TestCase):
    def test_template_tags_with_prefix(self):
        names = set_urls_for_scaffold(prefix='test')
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( names[attr], "{0}_generic_scaffold_testmodel_{1}".format(TestCrudManager.prefix, attr))
            
    def test_get_url_names_with_model(self):
        names = set_urls_for_scaffold(app='generic_scaffold', model='testmodel')
        for attr in ['list', 'create', 'update', 'delete', 'detail']:
            self.assertEquals( names[attr], "{0}_generic_scaffold_testmodel_{1}".format(TestCrudManager.prefix, attr))
            

class TestOverrideViews(TestCase):
    def setUp(self):
        self.crud = test_override_crud
        self.list_view = self.crud.get_list_class_view()
        self.create_view = self.crud.get_create_class_view()
        self.update_view = self.crud.get_update_class_view()
        self.delete_view = self.crud.get_delete_class_view()
        self.detail_view = self.crud.get_detail_class_view()

    def test_views_have_correct_parent_classes(self):
        self.assertEquals(self.list_view.__bases__[-1].__name__, "OverridenListView")
        self.assertEquals(self.create_view.__bases__[-1].__name__, "OverridenCreateView")
        self.assertEquals(self.update_view.__bases__[-1].__name__, "OverridenUpdateView")
        self.assertEquals(self.delete_view.__bases__[-1].__name__, "OverridenDeleteView")
        self.assertEquals(self.detail_view.__bases__[-1].__name__, "OverridenDetailView")


import django
from django.conf.urls import url

from django.core.urlresolvers import reverse
from django.views.generic import ListView, CreateView , DetailView, UpdateView, DeleteView, TemplateView
from six import with_metaclass
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model


def get_model_name(model):
    try:
        return model._meta.model_name
    except:
        return model._meta.module_name

def get_app_label(model):
    return model._meta.app_label
    

class CrudTracker(type):
    def __init__(cls, name, bases, attrs):

        try:
            if CrudManager not in bases:
                return
        except NameError:
            return
            
        try:
            cls.prefix
        except AttributeError:
            cls.prefix = None

        for r in CrudManager._registry:
            if r.prefix == cls.prefix:
                raise django.core.exceptions.ImproperlyConfigured
            
            if get_model_name(r.model)==get_model_name(cls.model) and \
                get_app_label(r.model) == get_app_label(cls.model):
                raise django.core.exceptions.ImproperlyConfigured
        CrudManager._registry.append(cls)


def identity(f):
    return f


class FallbackTemplateMixin(object, ):
    def get_template_names(self):
        names = super(FallbackTemplateMixin, self).get_template_names()
        if self.kind == 'delete':
            fallback_name = 'confirm_delete'
        elif self.kind in ['create', 'update']:
            fallback_name = 'form'
        else:
            fallback_name = self.kind 
        names.append('generic_scaffold/{0}.html'.format(fallback_name))
        return names
        
   

class CrudManager(with_metaclass(CrudTracker, object, )):
    _registry = []
    list_mixins = []
    delete_mixins = []
    detail_mixins = []
    create_mixins = []
    update_mixins = []
    
    list_view_class = ListView
    create_view_class = CreateView
    detail_view_class = DetailView
    update_view_class = UpdateView
    delete_view_class = DeleteView

    def __new__(cls):
        cls.app_label = get_app_label(cls.model)
        cls.model_name = get_model_name(cls.model)
            
        if hasattr(cls, 'prefix') and cls.prefix:
            cls.list_url_name = '{0}_{1}_{2}'.format(cls.prefix, cls.get_name(), 'list')
            cls.create_url_name = '{0}_{1}_{2}'.format(cls.prefix, cls.get_name(), 'create')
            cls.detail_url_name = '{0}_{1}_{2}'.format(cls.prefix, cls.get_name(), 'detail')
            cls.update_url_name = '{0}_{1}_{2}'.format(cls.prefix, cls.get_name(), 'update')
            cls.delete_url_name = '{0}_{1}_{2}'.format(cls.prefix, cls.get_name(), 'delete')
        else:
            cls.list_url_name = '{0}_{1}'.format(cls.get_name(), 'list')
            cls.create_url_name = '{0}_{1}'.format(cls.get_name(), 'create')
            cls.detail_url_name = '{0}_{1}'.format(cls.get_name(), 'detail')
            cls.update_url_name = '{0}_{1}'.format(cls.get_name(), 'update')
            cls.delete_url_name = '{0}_{1}'.format(cls.get_name(), 'delete')
            
        cls.model.list_url_name = cls.list_url_name
        cls.model.detail_url_name = cls.detail_url_name
        cls.model.create_url_name = cls.create_url_name
        cls.model.update_url_name = cls.update_url_name
        cls.model.delete_url_name = cls.delete_url_name
                
        return super(CrudManager, cls).__new__(cls)

    def __init__(self, *args, **kwargs):
        self.perms = {
            'list': identity,
            'create': identity,
            'update': identity,
            'delete': identity,
            'detail': identity,
        }
        if hasattr(self, 'permissions') and self.permissions:
            self.perms.update(self.permissions)

    def get_get_context_data(self, klazz, **kwargs):
        def wrapped_get_context_data(inst, **kwargs):
            context = super(klazz, inst).get_context_data(**kwargs)
            context['crud'] = self
            #context.update(inst.get_context_data() )
            return context
        return wrapped_get_context_data

    @classmethod
    def get_name(cls):
        try:
            model_name = cls.model._meta.model_name
        except:
            model_name = cls.model._meta.module_name
        return '{0}_{1}'.format(cls.model._meta.app_label, model_name)

    def get_list_class_view(self):
        name = '{0}_{1}'.format(self.get_name(), 'ListView')
        options_dict = {
            'kind': 'list',
            'model': self.model,
        }
        
        if hasattr(self, 'list_template_name') and self.list_template_name:
            options_dict['template_name'] = self.list_template_name

        parent_classes_list = [FallbackTemplateMixin]
        parent_classes_list.extend(self.list_mixins)
        parent_classes_list.append(self.list_view_class)

        klazz = type(name, tuple(parent_classes_list), options_dict )
        klazz.get_context_data = self.get_get_context_data(klazz)

        return klazz

    def get_create_class_view(self):
        name = '{0}_{1}'.format(self.get_name(), 'CreateView')
        options_dict = {
            'kind': 'create',
            'model': self.model,
            'fields': '__all__',
        }
        if hasattr(self, 'form_template_name') and self.form_template_name:
            options_dict['template_name'] = self.form_template_name
        
        if hasattr(self, 'form_class') and self.form_class:
            options_dict['form_class'] = self.form_class
            options_dict['fields'] = None

        parent_classes_list = [FallbackTemplateMixin]
        parent_classes_list.extend(self.create_mixins)
        parent_classes_list.append(self.create_view_class)
        
        klazz = type(name, tuple(parent_classes_list), options_dict )
        klazz.get_context_data = self.get_get_context_data(klazz)
        return klazz

    def get_detail_class_view(self):
        name = '{0}_{1}'.format(self.get_name(), 'DetailView')
        options_dict = {
            'kind': 'detail',
            'model': self.model,
        }
        if hasattr(self, 'detail_template_name') and self.detail_template_name:
            options_dict['template_name'] = self.detail_template_name
        
        parent_classes_list = [FallbackTemplateMixin]
        parent_classes_list.extend(self.detail_mixins)
        parent_classes_list.append(self.detail_view_class)

        klazz = type(name, tuple(parent_classes_list), options_dict )
        klazz.get_context_data = self.get_get_context_data(klazz)
        return klazz

    def get_update_class_view(self):
        name = '{0}_{1}'.format(self.get_name(), 'UpdateView')
        options_dict = {
            'kind': 'update',
            'model': self.model,
            'fields': '__all__',
        }
        if hasattr(self, 'form_template_name') and self.form_template_name:
            options_dict['template_name'] = self.form_template_name
        
        if hasattr(self, 'form_class') and self.form_class:
            options_dict['form_class'] = self.form_class
            options_dict['fields'] = None

        parent_classes_list = [FallbackTemplateMixin]
        parent_classes_list.extend(self.update_mixins)
        parent_classes_list.append(self.update_view_class)

        klazz = type(name, tuple(parent_classes_list), options_dict )
        klazz.get_context_data = self.get_get_context_data(klazz)
        return klazz

    def get_delete_class_view(self):
        name = '{0}_{1}'.format(self.get_name(), 'DeleteView')
        options_dict = {
            'model': self.model,
            'kind': 'delete',
            'fields': '__all__',
            'get_success_url': lambda x: reverse(self.list_url_name),
        }
        if hasattr(self, 'delete_template_name') and self.delete_template_name:
            options_dict['template_name'] = self.delete_template_name
        
        parent_classes_list = [FallbackTemplateMixin]
        parent_classes_list.extend(self.delete_mixins)
        parent_classes_list.append(self.delete_view_class)

        klazz = type(name, tuple(parent_classes_list), options_dict )
        klazz.get_context_data = self.get_get_context_data(klazz)
        return klazz

    def get_url_patterns(self, ):
        prefix = hasattr(self, 'prefix') and self.prefix or ''
        
        url_patterns = [
            url(r'^'+prefix+'$', self.perms['list'](self.get_list_class_view().as_view()), name=self.list_url_name, ),
            url(r'^'+prefix+'create/$', self.perms['create'](self.get_create_class_view().as_view()), name=self.create_url_name ),
            url(r'^'+prefix+'detail/(?P<pk>\d+)$', self.perms['detail'](self.get_detail_class_view().as_view()), name=self.detail_url_name ),
            url(r'^'+prefix+'update/(?P<pk>\d+)$', self.perms['update'](self.get_update_class_view().as_view()), name=self.update_url_name ),
            url(r'^'+prefix+'delete/(?P<pk>\d+)$', self.perms['delete'](self.get_delete_class_view().as_view()), name=self.delete_url_name ),
        ]
        
        if django.VERSION >= (1, 8, 0):
            return url_patterns
        else:
            from django.conf.urls import patterns
            return patterns('', *url_patterns)

    @classmethod
    def get_url_names(cls, prefix=None, model_class=None):
        for r in cls._registry:
            if r.prefix==prefix or model_class and r.model==model_class:
                return {
                    'list': r.list_url_name,
                    'create': r.create_url_name,
                    'update': r.update_url_name,
                    'delete': r.delete_url_name,
                    'detail': r.detail_url_name,
                }
                
def get_url_names(app=None, model=None, prefix=None):
    model_class = None
    if app and model: 
        model_class = get_model(app, model)
    return CrudManager.get_url_names(prefix=prefix, model_class=model_class)



from django import template
from django.conf import settings

from generic_scaffold import get_url_names

register = template.Library()

@register.assignment_tag
def set_urls_for_scaffold(app=None, model=None, prefix=None):
    url_names = get_url_names(app, model, prefix)
    return url_names

    



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='django-generic-scaffold',
    version='0.4.1',
    description='Generic scaffolding for Django',

    author='Serafeim Papastefanos',
    author_email='spapas@gmail.com',
    license='MIT',
    url='https://github.com/spapas/django-generic-scaffold/',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['tests.*', 'tests',]),

    install_requires=['Django >=1.6', 'six'],

    classifiers=[
        'Environment :: Web Environment',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
    ],
)
